"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Countdown {
    started;
    paused = undefined;
    durationMs;
    complete = false;
    timeoutID = undefined;
    callbackQueue = [];
    constructor(durationMs) {
        if (durationMs < 1) {
            throw new TypeError(`Invalid duration`);
        }
        this.started = Date.now();
        this.durationMs = durationMs;
        this.resume();
    }
    resume() {
        if (this.timeoutID !== undefined) {
            throw new TypeError(`Timer already running`);
        }
        if (this.complete) {
            throw new TypeError(`Timer already complete`);
        }
        this.timeoutID = setTimeout(() => {
            this._complete();
        }, this.paused === undefined
            ? this.durationMs
            : this.started + this.durationMs - this.paused);
        return this;
    }
    pause() {
        if (this.timeoutID === undefined) {
            throw new TypeError(`Timer already paused`);
        }
        if (this.complete) {
            throw new TypeError(`Timer already complete`);
        }
        this.paused = Date.now();
        clearTimeout(this.timeoutID);
        this.timeoutID = undefined;
        return this;
    }
    reset() {
        if (this.complete) {
            throw new TypeError(`Timer already complete`);
        }
        this.started = Date.now();
        if (this.paused !== undefined) {
            // Currently paused
            this.paused = Date.now();
        }
        else {
            // Currently running
            clearTimeout(this.timeoutID);
            this.timeoutID = setTimeout(() => {
                this._complete();
            }, this.durationMs);
        }
        return this;
    }
    then(callback) {
        this.callbackQueue.push(callback);
        return this;
    }
    _complete() {
        this.complete = true;
        this.callbackQueue.forEach((c) => c());
    }
}
exports.default = Countdown;
