"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const isMapLike_1 = __importDefault(require("./isMapLike"));
class Dict {
    [Symbol.toStringTag] = "xtjs-lib.Dict";
    map;
    // Allow Map-like objects so we can wrap StructuralMap instances and other objects that implement the Map interface.
    constructor(entriesOrMap) {
        this.map = (0, isMapLike_1.default)(entriesOrMap)
            ? entriesOrMap
            : entriesOrMap
                ? new Map(entriesOrMap)
                : new Map();
    }
    get size() {
        return this.map.size;
    }
    clone() {
        return new Dict(this.entries());
    }
    clear() {
        this.map.clear();
    }
    computeIfAbsent(key, provider) {
        if (!this.map.has(key)) {
            this.map.set(key, provider(key));
        }
        return this.map.get(key);
    }
    computeIf(key, pred, provider) {
        const cur = this.get(key);
        if (pred(cur)) {
            this.map.set(key, provider(key, cur));
        }
        return this.map.get(key);
    }
    containsKey(key) {
        return this.map.has(key);
    }
    entries() {
        return this.map.entries();
    }
    get(key) {
        return this.map.get(key);
    }
    getOrThrow(key, error) {
        if (!this.map.has(key)) {
            throw error?.(key) ?? new ReferenceError("Key does not exist");
        }
        return this.map.get(key);
    }
    getOrDefault(key, def) {
        return !this.map.has(key) ? def : this.map.get(key);
    }
    isEmpty() {
        return !this.map.size;
    }
    keys() {
        return this.map.keys();
    }
    put(key, value) {
        const old = this.map.get(key);
        this.map.set(key, value);
        return old;
    }
    putIfAbsentOrThrow(key, value, error) {
        if (this.map.has(key)) {
            throw (error?.(key, this.map.get(key)) ??
                new ReferenceError(`Key already exists`));
        }
        this.map.set(key, value);
        return value;
    }
    remove(key) {
        const old = this.map.get(key);
        this.map.delete(key);
        return old;
    }
    values() {
        return this.map.values();
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    delete(key) {
        return this.map.delete(key);
    }
    forEach(cb, thisArg) {
        return this.map.forEach(cb, thisArg);
    }
    has(key) {
        return this.map.has(key);
    }
    set(key, value) {
        this.map.set(key, value);
        return this;
    }
}
exports.default = Dict;
