"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_stream_1 = require("node:stream");
const assertIsBytes = (chunk, encoding) => {
    if (encoding !== "buffer" || !Buffer.isBuffer(chunk)) {
        console.error("Assertion failure: not bytes");
        throw new TypeError(`Expected bytes`);
    }
};
class Lines extends node_stream_1.Duplex {
    mapper;
    includeFinalUnterminated;
    buffer = Buffer.alloc(0);
    canPush = false;
    constructor(mapper = (line) => line, includeFinalUnterminated = false) {
        super({});
        this.mapper = mapper;
        this.includeFinalUnterminated = includeFinalUnterminated;
    }
    pushLine(upToPos) {
        // Include the LF in the value.
        const line = this.mapper(this.buffer.slice(0, upToPos).toString("utf-8"));
        this.buffer = this.buffer.slice(upToPos);
        if (line != undefined) {
            this.canPush = this.push(line);
        }
    }
    maybeRead() {
        while (this.canPush) {
            const pos = this.buffer.indexOf(10);
            if (pos == -1) {
                break;
            }
            this.pushLine(pos + 1);
        }
    }
    _read() {
        this.canPush = true;
        this.maybeRead();
    }
    _write(chunk, encoding, callback) {
        assertIsBytes(chunk, encoding);
        this.buffer = Buffer.concat([this.buffer, chunk]);
        callback();
        this.maybeRead();
    }
    _writev(chunks, callback) {
        chunks.forEach((c) => assertIsBytes(c.chunk, c.encoding));
        this.buffer = Buffer.concat([this.buffer, ...chunks.map((c) => c.chunk)]);
        callback();
        this.maybeRead();
    }
    _final(callback) {
        if (this.includeFinalUnterminated && this.buffer.length) {
            this.pushLine(this.buffer.length);
        }
        callback();
    }
}
exports.default = Lines;
