"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assertExists_1 = __importDefault(require("./assertExists"));
const assertState_1 = __importDefault(require("./assertState"));
const describeType_1 = __importDefault(require("./describeType"));
const LinkedList_1 = __importDefault(require("./LinkedList"));
// FIFO pool: the oldest available will be repeatedly returned until marked unavailable.
class Pool {
    available = new LinkedList_1.default();
    unavailable = new LinkedList_1.default();
    // In case items are in multiple Pool instances.
    itemStatePropSymbol = Symbol("kPoolItemState");
    getStateOfItem(item) {
        return (0, assertExists_1.default)(item[this.itemStatePropSymbol], "Item was never registered in this pool");
    }
    getAvailable() {
        return this.available.getHead()?.getValue();
    }
    makeAvailable(item) {
        const state = this.getStateOfItem(item);
        if (!state.available) {
            state.available = true;
            state.node.detach();
            state.node = this.available.append(item);
        }
    }
    makeUnavailable(item) {
        const state = this.getStateOfItem(item);
        if (state.available) {
            state.available = false;
            state.node.detach();
            state.node = this.unavailable.append(item);
        }
    }
    registerNewItem(item) {
        (0, assertState_1.default)(!!item && typeof item == "object", `Pools can only take objects (got ${(0, describeType_1.default)(item)})`);
        (0, assertState_1.default)(!(this.itemStatePropSymbol in item), `Item has already been registered`);
        const node = this.available.append(item);
        const state = {
            node,
            available: true,
        };
        item[this.itemStatePropSymbol] = state;
    }
    deregisterItem(item) {
        if (this.itemStatePropSymbol in item) {
            const state = this.getStateOfItem(item);
            state.node.detach();
            delete item[this.itemStatePropSymbol];
        }
    }
}
exports.default = Pool;
