import Comparator from "./Comparator";
export declare class ResultUnwrapError extends Error {
}
export default class Result<T, E> {
    private readonly pair;
    private constructor();
    static Ok<T, E>(result: T): Result<T, E>;
    static Err<T, E>(error: E): Result<T, E>;
    get result(): T | undefined;
    get error(): E | undefined;
    readonly and: <U>(other: Result<U, E>) => Result<U, E>;
    readonly andThen: <U>(other: (val: T) => Result<U, E>) => Result<U, E>;
    readonly contains: (other: T, comparator?: Comparator<T>) => boolean;
    readonly containsErr: (other: E, comparator?: Comparator<E>) => boolean;
    readonly expect: (msg: string) => T;
    readonly expectErr: (msg: string) => E;
    readonly flatten: () => T extends Result<infer U, infer F> ? Result<U, F> : Result<T, E>;
    readonly inspect: (fn: (val: T) => unknown) => Result<T, E>;
    readonly inspectErr: (fn: (val: E) => unknown) => Result<T, E>;
    readonly isErr: () => boolean;
    readonly isErrAnd: (pred: (err: E) => any) => any;
    readonly map: <U>(op: (val: T) => U) => Result<U, E>;
    readonly mapErr: <F>(op: (val: E) => F) => Result<T, F>;
    readonly mapOr: <U>(def: U, op: (val: T) => U) => U;
    readonly mapOrElse: <U>(def: (val: E) => U, op: (val: T) => U) => U;
    readonly or: <F>(other: Result<T, F>) => Result<T, F>;
    readonly orElse: <F>(other: (err: E) => Result<T, F>) => Result<T, F>;
    readonly unwrap: () => T;
    readonly unwrapErr: () => E;
    readonly unwrapOr: (def: T) => T;
    readonly unwrapOrElse: (def: (err: E) => T) => T;
    [Symbol.iterator](): Generator<T | E | undefined, void, unknown>;
}
export declare const Ok: <T, E>(val: T) => Result<T, E>;
export declare const Err: <T, E>(err: E) => Result<T, E>;
