"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Err = exports.Ok = exports.ResultUnwrapError = void 0;
const AssertionError_1 = __importDefault(require("./AssertionError"));
const nativeOrdering_1 = __importDefault(require("./nativeOrdering"));
const type_1 = __importDefault(require("./type"));
class ResultUnwrapError extends Error {
}
exports.ResultUnwrapError = ResultUnwrapError;
// `E` should never be false-like.
// Patterns:
// const [err, res] = resultable();
// const {error, result} = resultable();
// try { resultable().unwrap() } catch (err) { ... }
class Result {
    pair;
    constructor(pair) {
        this.pair = pair;
    }
    static Ok(result) {
        return new Result([true, result]);
    }
    static Err(error) {
        return new Result([false, error]);
    }
    get result() {
        return this.pair[0] ? this.pair[1] : undefined;
    }
    get error() {
        return this.pair[0] ? undefined : this.pair[1];
    }
    and = (other) => {
        return this.pair[0] ? other : Result.Err(this.pair[1]);
    };
    andThen = (other) => {
        return this.pair[0] ? other(this.pair[1]) : Result.Err(this.pair[1]);
    };
    contains = (other, comparator = nativeOrdering_1.default) => {
        return this.pair[0] && comparator(this.pair[1], other) === 0;
    };
    containsErr = (other, comparator = nativeOrdering_1.default) => {
        return !this.pair[0] && comparator(this.pair[1], other) === 0;
    };
    expect = (msg) => {
        if (!this.pair[0]) {
            throw new AssertionError_1.default(msg);
        }
        return this.pair[1];
    };
    expectErr = (msg) => {
        if (this.pair[0]) {
            throw new AssertionError_1.default(msg);
        }
        return this.pair[1];
    };
    flatten = () => {
        if (this.pair[0] && (0, type_1.default)(this.pair[1]) == Result) {
            return this.pair[1];
        }
        return this;
    };
    inspect = (fn) => {
        if (this.pair[0]) {
            fn(this.pair[1]);
        }
        return this;
    };
    inspectErr = (fn) => {
        if (!this.pair[0]) {
            fn(this.pair[1]);
        }
        return this;
    };
    isErr = () => {
        return !this.pair[0];
    };
    isErrAnd = (pred) => {
        return !this.pair[0] && pred(this.pair[1]);
    };
    map = (op) => {
        return !this.pair[0]
            ? Result.Err(this.pair[1])
            : Result.Ok(op(this.pair[1]));
    };
    mapErr = (op) => {
        return this.pair[0]
            ? Result.Ok(this.pair[1])
            : Result.Err(op(this.pair[1]));
    };
    mapOr = (def, op) => {
        return !this.pair[0] ? def : op(this.pair[1]);
    };
    mapOrElse = (def, op) => {
        return !this.pair[0] ? def(this.pair[1]) : op(this.pair[1]);
    };
    or = (other) => {
        return !this.pair[0] ? other : Result.Ok(this.pair[1]);
    };
    orElse = (other) => {
        return !this.pair[0] ? other(this.pair[1]) : Result.Ok(this.pair[1]);
    };
    unwrap = () => {
        if (!this.pair[0]) {
            throw this.pair[1];
        }
        return this.pair[1];
    };
    unwrapErr = () => {
        if (this.pair[0]) {
            throw new ResultUnwrapError("Called unwrapErr on Ok");
        }
        return this.pair[1];
    };
    unwrapOr = (def) => {
        return !this.pair[0] ? def : this.pair[1];
    };
    unwrapOrElse = (def) => {
        return !this.pair[0] ? def(this.pair[1]) : this.pair[1];
    };
    *[Symbol.iterator]() {
        if (this.pair[0]) {
            yield undefined;
            yield this.pair[1];
        }
        else {
            yield this.pair[1];
            yield undefined;
        }
    }
}
exports.default = Result;
const Ok = (val) => Result.Ok(val);
exports.Ok = Ok;
const Err = (err) => Result.Err(err);
exports.Err = Err;
