"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const findAndRemove_1 = __importDefault(require("./findAndRemove"));
class Pair {
    key;
    value;
    constructor(key, value) {
        this.key = key;
        this.value = value;
    }
}
const EMPTY_ARRAY = [];
class StructuralMap {
    [Symbol.toStringTag] = "xtjs-lib.StructuralMap";
    map = new Map();
    _size = 0;
    get size() {
        return this._size;
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    clear() {
        this.map.clear();
        this._size = 0;
    }
    delete(key) {
        const hc = key.hashCode();
        const pairs = this.map.get(hc) ?? EMPTY_ARRAY;
        const deleted = (0, findAndRemove_1.default)(pairs, (p) => key.equals(p.key));
        if (deleted) {
            this._size--;
        }
        return !!deleted;
    }
    *entries() {
        for (const pairs of this.map.values()) {
            for (const pair of pairs) {
                yield [pair.key, pair.value];
            }
        }
    }
    forEach(callbackfn, thisArg) {
        for (const [key, value] of this.entries()) {
            callbackfn.call(thisArg, value, key, this);
        }
    }
    get(key) {
        const hc = key.hashCode();
        const pairs = this.map.get(hc) ?? EMPTY_ARRAY;
        return pairs.find((p) => key.equals(p.key))?.value;
    }
    has(key) {
        const hc = key.hashCode();
        const pairs = this.map.get(hc) ?? EMPTY_ARRAY;
        return !!pairs.find((p) => key.equals(p.key));
    }
    *keys() {
        for (const pairs of this.map.values()) {
            for (const pair of pairs) {
                yield pair.key;
            }
        }
    }
    set(key, value) {
        const hc = key.hashCode();
        let pairs = this.map.get(hc);
        if (!pairs) {
            this.map.set(hc, (pairs = []));
        }
        const pair = pairs.find((p) => key.equals(p.key));
        if (!pair) {
            pairs.push(new Pair(key, value));
            this._size++;
        }
        else {
            pair.value = value;
        }
        return this;
    }
    *values() {
        for (const pairs of this.map.values()) {
            for (const pair of pairs) {
                yield pair.value;
            }
        }
    }
}
exports.default = StructuralMap;
