"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assertState_1 = __importDefault(require("./assertState"));
const mapExists_1 = __importDefault(require("./mapExists"));
class YearMonth {
    year;
    month;
    constructor(year, month) {
        this.year = year;
        this.month = month;
        (0, assertState_1.default)(Number.isSafeInteger(year) && year >= 0 && year <= 9999);
        (0, assertState_1.default)(Number.isSafeInteger(month) && month >= 1 && month <= 12);
    }
    static ofDate(dt) {
        return new YearMonth(dt.getFullYear(), dt.getMonth() + 1);
    }
    static parse(raw) {
        return (0, mapExists_1.default)(/^([0-9]{4})(0[1-9]|1[0-2])$/.exec(raw), ([_, y, m]) => new YearMonth(+y, +m));
    }
    static now() {
        return YearMonth.ofDate(new Date());
    }
    startUtc() {
        return new Date(Date.UTC(this.year, this.month - 1));
    }
    // WARNING: This is exclusive.
    endUtc() {
        return new Date(Date.UTC(this.year, this.month + 1 - 1));
    }
    toJSON() {
        return this.toString();
    }
    toString() {
        return [
            this.year.toString().padStart(4, "0"),
            this.month.toString().padStart(2, "0"),
        ].join("");
    }
}
exports.default = YearMonth;
