"use strict";
// This is a simpler version of @xtjs/lang/ts/cancellable.ts without needing special syntax and code transformers while remaining type safe and ergonomic. It uses standard AbortController/AbortSignal and Promise APIs instead of custom types and functions.
// We use the term "chain" as "pipe" is overloaded.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManagedAbortableChain = void 0;
class AbortableChain {
    abortSignal;
    stages = Array();
    constructor(
    // For simplicity and consistency with other APIs that use AbortController/AbortSignal, the controller should be external and only the signal should be provided. Aborting is now done externally, which avoids us having to return a Promise-like + abort() object when calling exec().
    abortSignal) {
        this.abortSignal = abortSignal;
    }
    add(fn) {
        this.stages.push(fn);
        return this;
    }
    async exec() {
        let out = undefined;
        for (const stage of this.stages) {
            this.abortSignal.throwIfAborted();
            out = await stage(out, this.abortSignal);
        }
        return out;
    }
}
exports.default = AbortableChain;
class ManagedAbortableChain extends AbortableChain {
    controller;
    constructor() {
        const controller = new AbortController();
        super(controller.signal);
        this.controller = controller;
    }
    add(fn) {
        return super.add(fn);
    }
    async exec() {
        const promise = super.exec();
        // Don't return Promise as custom properties set on it like `abort` won't stick.
        const res = {
            [Symbol.toStringTag]: ManagedAbortableChain.name,
            then: (f, r) => promise.then(f, r),
            catch: (r) => promise.catch(r),
            finally: (n) => promise.finally(n),
            abort: (reason) => this.controller.abort(reason),
        };
        return res;
    }
}
exports.ManagedAbortableChain = ManagedAbortableChain;
