"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assertExists_1 = __importDefault(require("./assertExists"));
const assertState_1 = __importDefault(require("./assertState"));
const concatBuffers_1 = __importDefault(require("./concatBuffers"));
// Takes in Uint8Array instances of various lengths and outputs Uint8Array instances of a fixed length.
class Chunker {
    size;
    buf = Array();
    bufSize = 0;
    constructor(size) {
        this.size = size;
    }
    *push(bytes, chunkSizeOverride = this.size) {
        const byteCount = bytes.byteLength;
        this.buf.push(bytes);
        this.bufSize += byteCount;
        const parts = [];
        let partLen = 0;
        while (this.bufSize >= chunkSizeOverride - partLen) {
            const part = (0, assertExists_1.default)(this.buf[0]);
            // How many bytes would be remaining if we added the first Uint8Array in the buffer.
            const rem = partLen + part.byteLength - chunkSizeOverride;
            if (rem <= 0) {
                // If rem < 0: adding this Uint8Array would still not be enough, so add this part and continue the loop.
                // If rem == 0: adding this Uint8Array would be exactly enough for a single chunk.
                parts.push(part);
                partLen += part.byteLength;
                this.buf.shift();
                this.bufSize -= part.byteLength;
                if (rem == 0) {
                    yield (0, concatBuffers_1.default)(parts.splice(0), partLen);
                    partLen = 0;
                }
            }
            else {
                const partPart = part.slice(0, -rem);
                parts.push(partPart);
                yield (0, concatBuffers_1.default)(parts.splice(0), chunkSizeOverride);
                this.bufSize -= partPart.byteLength;
                this.buf[0] = part.slice(-rem);
                partLen = 0;
            }
        }
        (0, assertState_1.default)(!parts.length);
        (0, assertState_1.default)(partLen === 0);
        return;
    }
    takeRemaining() {
        const { bufSize } = this;
        const data = (0, concatBuffers_1.default)(this.buf.splice(0), bufSize);
        this.bufSize = 0;
        return data;
    }
}
exports.default = Chunker;
