"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// This is like `select!` in other languages. We can't use Promise.race since it doesn't clean up the loser promises (promises don't have cancel/cleanup functionality).
// WARNING: This doesn't check if it's possible for the events to even emit anymore. Check the state of the event emitters before calling this function. Alternatively, use a timeout.
// Tip: to set a timeout, provide an AbortSignal.timeout() as one of the emitters.
exports.default = () => {
    const emitters = Array();
    const builder = {
        add(emitter, handlers) {
            emitters.push({ emitter, handlers });
            return builder;
        },
        wait() {
            return new Promise((resolve, reject) => {
                const cleanUps = Array();
                for (const { emitter, handlers } of emitters) {
                    for (const [eventName, cb] of Object.entries(handlers)) {
                        const handler = (arg) => {
                            for (const fn of cleanUps) {
                                fn();
                            }
                            try {
                                resolve(cb(arg));
                            }
                            catch (err) {
                                reject(err);
                            }
                        };
                        cleanUps.push(() => {
                            if ("removeEventListener" in emitter) {
                                emitter.removeEventListener(eventName, handler);
                            }
                            else {
                                emitter.off(eventName, handler);
                            }
                        });
                        if ("addEventListener" in emitter) {
                            emitter.addEventListener(eventName, handler);
                        }
                        else {
                            emitter.on(eventName, handler);
                        }
                    }
                }
            });
        },
    };
    return builder;
};
